#!/usr/bin/env python
import re, sys

def open_file():
    if len(sys.argv) >= 2:
        trace_file = sys.argv[1]
        return open(trace_file)
    else:
        return sys.stdin

NUM = '(0x[0-9a-f]+|[0-9]+)'
pci_line = re.compile(' bus=' + NUM + ' devfn=' + NUM)

bus_map = {}
bus_guess_map = {
    (0x01, 0): 0x00,    # PCI bridge [0604]: Intel Corporation Skylake PCIe Controller (x16) [8086:1901] (rev 07)
    (0x02, 0): 0x00,    # VGA compatible controller [0300]: Intel Corporation HD Graphics 530 [8086:191b] (rev 06)
    (0x1f, 0): 0x00,    # ISA bridge [0601]: Intel Corporation Sunrise Point-H LPC Controller [8086:a14e] (rev 31)
    (0x1f, 2): 0x00,    # Memory controller [0580]: Intel Corporation Sunrise Point-H PMC [8086:a121] (rev 31)
    (0x1f, 3): 0x00,    # Audio device [0403]: Intel Corporation Sunrise Point-H HD Audio [8086:a170] (rev 31)
    (0x1f, 4): 0x00,    # SMBus [0c05]: Intel Corporation Sunrise Point-H SMBus [8086:a123] (rev 31)
    (0x00, 0): 0x01,    # VGA compatible controller [0300]: NVIDIA Corporation GM204M [GeForce GTX 965M] [10de:13d9] (rev a1)
}
def find_bus(bus_addr, dev, fn):
    bus = bus_map.get(bus_addr)
    if bus is None:
        bus = bus_guess_map.get((dev, fn))
        if bus is not None:
            # Prevent reuse
            bus_map[bus_addr] = bus
            del bus_guess_map[(dev, fn)]
    return bus

def fixup_device(match):
    bus_addr = int(match.group(1), 0)
    devfn = int(match.group(2), 0)
    dev, fn = (devfn >> 3), (devfn & 7)
    bus = find_bus(bus_addr, dev, fn)
    if bus is not None:
        return ' dev=%02x:%02x:%o' % (bus, dev, fn)
    return match.group(0)

with open_file() as f:
    for line in f:
        line = pci_line.sub(fixup_device, line)
        sys.stdout.write(line)

# Show all resolved addresses
#for addr, bus in bus_map.items():
#    print("%x:\t%02x" % (addr, bus))
